(ns example.core
  (:require [com.stuartsierra.component :as component]
            [cljs.core.async :refer [<! >! put! chan] :as async]
            [taoensso.sente  :as sente :refer [cb-success?]]
            )
  (:require-macros [cljs.core.async.macros :refer [go go-loop] :as asyncm]))

(.info js/console "Prod loaded.")

(defrecord App [sente]
  component/Lifecycle
  (start [component]
    (.info js/console "App started. sente=" sente)
    component)
  (stop [component]
    (.info js/console "App stopped.")
    component))

(defrecord Sente [client-id
                  sente-chsk]
  component/Lifecycle
  (start [c]
    (let [sente-chsk (sente/make-channel-socket! "/chsk" {:type :auto
                                                          :client-id client-id})]
      (go (while true
            (let [event (<! (:ch-recv sente-chsk))]
              (.info js/console (str "Got event! " event)))))
      (.info js/console "Sente started.")

      (.setTimeout js/window #((:send-fn sente-chsk) [:client/hi]) 500)

      (assoc c :sente-chsk sente-chsk)))
  (stop [c]
    (when (:sente-chsk c)
      (.info js/console "Sente stopped.")
      (dissoc c :sente-chsk))))

(defonce system (atom (component/system-map :sente (map->Sente {:client-id "1234"})
                                            :app (component/using
                                                   (map->App {})
                                                   {:sente :sente}))))

(defn send-message [msg]
  (let [sente-chsk (get-in @system [:sente :sente-chsk])]
    ))

(defn start! [window]
  (.info js/console "Starting...")
  (swap! system component/start)
  (.info js.console "Finished starting."))